#ifndef A_BROWSERWIDGET2_H
#define A_BROWSERWIDGET2_H

#include "ui_browserwidget2.h"
#include <QApplication>
#include <QWidget>
#include <QModelIndex>
#include <QDropEvent>
#include <QDragEnterEvent>
#include <QDragMoveEvent>
#include <QDragLeaveEvent>
#include <QCloseEvent>
#include <QIcon>
#include <QSettings>
#include <QShortcut>
#include <QProgressDialog>
#include <set>
#include <mdcmTag.h>
#include <mdcmVL.h>
#include <mdcmSimpleSubjectWatcher.h>
#include <mdcmEvent.h>
#include <mdcmDataSet.h>
#include <mdcmDict.h>

class ScannerWatcher : public mdcm::SimpleSubjectWatcher
{
public:
	ScannerWatcher(mdcm::Subject * s, const char * comment = "")
		: mdcm::SimpleSubjectWatcher(s, comment) {}
	void StartFilter() override
	{
		QApplication::processEvents();
	}
	void EndFilter() override
	{
		QApplication::processEvents();
	}
	void ShowProgress(mdcm::Subject*, const mdcm::Event&) override
	{
		QApplication::processEvents();
	}
	void ShowFileName(mdcm::Subject*, const mdcm::Event&) override {}
};

class EntryDICOMDIR
{
public:
	EntryDICOMDIR() = default;
	~EntryDICOMDIR() = default;
	unsigned int offsetOfTheNextDirectoryRecord{};
	unsigned int offsetOfReferencedLowerLevelDirectoryEntity{};
	QString directoryRecordType;
	QString modality;
	QString patient;
	QString birthdate;
	QString study_date;
	QString study_desc;
	QString series_date;
	QString series_desc;
	QString file;
};

class SeriesDICOMDIR
{
public:
	SeriesDICOMDIR() = default;
	~SeriesDICOMDIR() = default;
	bool    eye{};
	bool    eye2{};
	QString UID;
	QString modality;
	QString patient;
	QString birthdate;
	QString study;
	QString study_date;
	QString series;
	QString series_date;
	QStringList files;
};

class TableWidgetItem : public QTableWidgetItem
{
public:
	TableWidgetItem()
		: QTableWidgetItem(QTableWidgetItem::UserType + 1) {}
	TableWidgetItem(const QString & s)
		: QTableWidgetItem(s, QTableWidgetItem::UserType + 1) {}
	~TableWidgetItem() = default;
	QStringList files;
};

class BrowserWidget2: public QWidget, public Ui::BrowserWidget2
{
Q_OBJECT
public:
	BrowserWidget2(float);
	~BrowserWidget2() = default;
	bool          is_first_run() const;
	const QString read_DICOMDIR(const QString&);
	QStringList   get_files_of_1st();
	void          writeSettings(QSettings&);
	QString       get_root() const;

protected:
	void closeEvent(QCloseEvent*) override;
	void dropEvent(QDropEvent*) override;
	void dragEnterEvent(QDragEnterEvent*) override;
	void dragMoveEvent(QDragMoveEvent*) override;
	void dragLeaveEvent(QDragLeaveEvent*) override;
	void readSettings();

public slots:
	void read_directory(const QString&);
	void open_dicom_dir2(const QString&);
	void open_DICOMDIR2(const QString&);
	void reload_dir();
	void open_dicom_dir();
	void copy_files();
	void open_DICOMDIR();
#ifdef USE_WORKSTATION_MODE
	void open_CTK_db();
#endif

private:
	bool once{};
	QString saved_copy_dir;
	QIcon eye_icon;
	QIcon eye2_icon;
	std::set<mdcm::Tag> selected_tags;
	std::set<mdcm::Tag> selected_tags_short;
	mdcm::VL compute_offset0(const mdcm::DataSet&);
	void compute_offsets(
		const mdcm::SequenceOfItems*,
		mdcm::VL,
		std::vector<unsigned int> &);
	void process_directory(
		const QString&,
		const mdcm::Dict&,
		QProgressDialog*);
	unsigned int add_roots(
		const QMap<unsigned int, EntryDICOMDIR> &,
		unsigned int,
		QList<SeriesDICOMDIR> &,
		bool*);
	unsigned int add_study(
		const QMap<unsigned int, EntryDICOMDIR> &,
		unsigned int,
		QList<SeriesDICOMDIR> &,
		const QString&,
		const QString&,
		bool*);
	unsigned int add_series(
		const QMap<unsigned int, EntryDICOMDIR> &,
		unsigned int,
		QList<SeriesDICOMDIR> &,
		const QString&,
		const QString&,
		const QString&,
		const QString&,
		bool*);
	unsigned int add_file(
		const QMap<unsigned int, EntryDICOMDIR> &,
		unsigned int,
		SeriesDICOMDIR&);
	void read_tags_(
		const QString &,
		QString&,QString&,QString&,
		QString&,QString&,QString&,QString&,
		bool*,bool*);
	void read_tags_short_(const QString&, bool*, bool*);
#ifdef USE_WORKSTATION_MODE
	QString ctk_dir;
	QString ctk_pname;
	QString ctk_pid;
	QString ctk_from;
	QString ctk_to;
	bool    ctk_apply_range;
#endif
	QShortcut * refresh_sc;
};

#endif

